'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _FetchState = require('../FetchState');

var FetchState = _interopRequireWildcard(_FetchState);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Created by leiyouwho on 6/29/16.
 */

var isChangeAction = function isChangeAction(actionString) {
  if (actionString.substr(0, 9) === '@@CHANGE/') {
    return true;
  }
  return false;
};

var isDeleteAction = function isDeleteAction(actionString) {
  if (actionString.substr(0, 9) === '@@DELETE/') {
    return true;
  }
  return false;
};

var isPushAction = function isPushAction(actionString) {
  if (actionString.substr(0, 7) === '@@PUSH/') {
    return true;
  }
  return false;
};

var getPathArray = function getPathArray(actionString, reducerPathRE) {
  var index = actionString.indexOf('/');
  if (index !== -1) {
    var reducerPathString = actionString.substring(index + 1);
    reducerPathString = reducerPathString.replace(reducerPathRE, '');
    return reducerPathString.split('/');
  }
};

exports.default = function (handlers, defaultState, reducerPathRE) {
  return function () {
    var state = arguments.length <= 0 || arguments[0] === undefined ? defaultState : arguments[0];
    var action = arguments[1];
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = handlers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var handler = _step.value;

        if (handler.type === action.type) {
          return handler.successCallBack(state, action);
        } else if (FetchState.REQUEST(handler.type) === action.type) {
          return handler.requestCallBack(state, action);
        } else if (FetchState.SUCCESS(handler.type) === action.type) {
          return handler.successCallBack(state, action);
        } else if (FetchState.FAILURE(handler.type) === action.type) {
          return handler.failureCallBack(state, action);
        } else if (handler.type instanceof Error) {
          return handler.catchCallBack(state, action);
        } else if (reducerPathRE.test(action.type)) {
          var statePath = action.type.replace(reducerPathRE, '');
          console.log(statePath.split('/'));
          var result = state.setIn(statePath.split('/'), action.data);
          return result;
        } else if (isChangeAction(action.type)) {
          return state.setIn(getPathArray(action.type, reducerPathRE), action.data);
        } else if (isDeleteAction(action.type)) {
          console.log('delete: ', getPathArray(action.type, reducerPathRE));
          return state.deleteIn(getPathArray(action.type, reducerPathRE));
        } else if (isPushAction(action.type)) {
          return state.setIn(getPathArray(action.type, reducerPathRE), state.getIn(getPathArray(action.type, reducerPathRE)).push(action.data));
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    return state;
  };
};

// import Immutable from 'immutable';
// import * as FetchState from '../FetchState';
//
// export default (handlers, defaultState, reducerPathRE) => (state = defaultState, action) => {
//   for (const handler of handlers) {
//     if (handler.type === action.type) {
//       console.log(action.type, JSON.stringify(state));
//       return handler.successCallBack(state, action);
//     } else if (handler.type instanceof Error) {
//       return handler.catchCallBack(state, action);
//     }
//     else if (reducerPathRE.test(action.type)) {
//       const statePath = action.type.replace(reducerPathRE, "");
//       const result = state.setIn(statePath.split('/'), action.data);
//       return result;
//       // var a = {
//       //   a: 'a',
//       //   b: {
//       //     b: 'b'
//       //   }
//       // }
//       // var b = Immutable.fromJS(a);
//       // console.log(b);
//       // for (const key of statePath.split('/')) {
//       //   console.log('key ', key);
//       // }
//       return state;
//     }
//   }
//     // else if ('StoryBoardReducer/components/0/props/title' === action.type) {
//     //   // let components = state.get('components');
//     //   // let component = components.get("0");
//     //   // let props = component.get('props');
//     //   // props = Object.assign({}, props, {title: action.data});
//     //   // component = component.set('props', props);
//     //   // components = components.set('0', component);
//     //   // return state.set('components', components);
//     //   // return state;
//     //   const statePath = action.type.replace(reducerPathRE, "");
//     //   const pathArr = statePath.split('/');
//     //
//     //   let result;
//     //   const setState = (state, path, value) => {
//     //     let subStatePath = path;
//     //     let key = subStatePath[0];
//     //     let subState = state;
//     //     console.error(subStatePath.length, key);
//     //     try {
//     //       subState = subState.get(key);
//     //     } catch(e) {
//     //       if (subStatePath.length === 1) {
//     //         try {
//     //           subState = subState.set(key, value);
//     //         } catch(e) {
//     //           // subState[key] = value;
//     //           console.log('subState, length1',subState);
//     //           subState = Object.assign({}, subState, {[key]: value});
//     //           console.log('subState, length1',subState);
//     //         }
//     //       } else if (subStatePath.length === 0) {
//     //         return;
//     //       }
//     //       // if (typeof subState === 'object') {
//     //       //   subState = Object.assign({}, subState, {[key]: value});
//     //       //   console.log(subState, key);
//     //       // } else {
//     //       //   subState = subState[key];
//     //       //   console.log(subState);
//     //       // }
//     //     }
//     //
//     //     if (subStatePath.length === 1) {
//     //       try {
//     //         subState = subState.set(key, value);
//     //       } catch(e) {
//     //         // subState[key] = value;
//     //         subState = Object.assign({}, subState, {[key]: value});
//     //       }
//     //     } else if (subStatePath.length === 0) {
//     //       return;
//     //     }
//     //
//     //
//     //       subStatePath = subStatePath.slice(1);
//     //     key = subStatePath[0];
//     //     console.warn('result', JSON.stringify(subState));
//     //     setState(subState, subStatePath, value);
//     //     result = subState;
//     //     console.log('result', JSON.stringify(result));
//     //   };
//       //
//       // setState(state, pathArr, action.data);
//       // console.warn('state', state);
//       // console.warn('result', JSON.stringify(result));
//       // console.warn('return', JSON.stringify(state.set(pathArr[0], result)));
//       // return state.set(pathArr[0], result);
//       // // try {
//       // //   return state.set(pathArr[0], result);
//       // // } catch (e) {
//       // //   return state[pathArr[0]] = result;
//       // // }
//       //
//       //
//       // // pathArr.map(key => {
//       // //   console.warn(key);
//       // //   console.log(JSON.stringify(state));
//       // //   let muteState = state;
//       // //   try {
//       // //   } catch(e) {
//       // //
//       // //   }
//       // // });
//       // return state;
//     // }
//   return state;
// };


module.exports = exports['default'];